----------------------------------------
-- _Whisper
----------------------------------------

GroupCalendar._Whisper = {}
GroupCalendar._Whisper.Commands = {}

function GroupCalendar._Whisper:Construct()
	MCEventLib:RegisterEvent("PLAYER_ENTERING_WORLD", self.PlayerEnteringWorld, self)
	self.ResponseQueue = {}
end

function GroupCalendar._Whisper:ChatFrame_MessageEventHandler(pEvent, ...)
	if pEvent == "CHAT_MSG_WHISPER" then
		local vMessage = strlower(arg1)
		
		if vMessage == "!gc" or strsub(vMessage, 1, 4) == "!gc " then
			return
		end
	elseif pEvent == "CHAT_MSG_WHISPER_INFORM" then
		if arg2 == GroupCalendar.PlayerName
		and strsub(arg1, 1, 4) == "<GC>" then
			return
		end
	end
	
	return self.Orig_ChatFrame_MessageEventHandler(pEvent, ...)
end

function GroupCalendar._Whisper:ChatMessageWhisper(pEvent, ...)
	local vMessage = strlower(arg1)
	local vSender = arg2
	
	-- Ignore it if it isn't a gc request
	
	if vMessage ~= "!gc"
	and strsub(vMessage, 1, 4) ~= "!gc " then
		return
	end
	
	-- Ignore them if they're not supposed to be whispering us for calendars
	
	if not GroupCalendar.Network:UserIsTrusted(vSender, "RAT") then
		self:WhisperResponse(pPlayerName, "You are not permitted to query this calendar")
	end
	
	-- Figure out what they want
	
	vMessage = string.sub(vMessage, 5)
	
	local vStartIndex, vEndIndex, vCommand, vParameter = string.find(vMessage, "([^%s]+) ?(.*)")
	
	if not vCommand then
		vCommand = "SUMMARY"
	end
	
	vCommand = string.upper(vCommand)
	
	if self.Commands[vCommand] then
		self.Commands[vCommand](self, vSender, vParameter)
	else
		self:WhisperResponse(vSender, string.format(GroupCalendar.cWhisperInvalidOption, vCommand))
	end
end

function GroupCalendar._Whisper:GetEventSummary(pUserName)
	local vCurrentDate, vCurrentTime = MCDateLib:GetServerDateTime()
	
	if vCurrentDate ~= self.CurrentDate then
		GroupCalendar.DeleteTable(self.EventSummary)
		
		self.EventSummary = GroupCalendar.Database.GetCompiledSchedule(vCurrentDate - 1, false, true)
		
		for vDateOffset = 0, 7 do
			self.EventSummary = GroupCalendar.Database.GetCompiledSchedule(vCurrentDate + vDateOffset, false, true, self.EventSummary)
		end
	end
	
	return self.EventSummary
end

function GroupCalendar._Whisper:PlayerEnteringWorld()
	self:HookChatFrame()
end

function GroupCalendar._Whisper:HookChatFrame()
	if self.Orig_ChatFrame_MessageEventHandler then
		return
	end
	
	self.Orig_ChatFrame_MessageEventHandler = ChatFrame_MessageEventHandler
	ChatFrame_MessageEventHandler = function (pEvent, ...) return GroupCalendar.Whisper:ChatFrame_MessageEventHandler(pEvent, ...) end
	
	MCEventLib:RegisterEvent("CHAT_MSG_WHISPER", self.ChatMessageWhisper, self)
end

function GroupCalendar._Whisper:WhisperResponse(pPlayerName, pMessage, ...)
	if select("#", ...) > 0 then
		pMessage = string.format(pMessage, ...)
	end
	
	table.insert(self.ResponseQueue, {Name = pPlayerName, Message = pMessage})
	
	if #self.ResponseQueue == 1 then
		MCSchedulerLib:ScheduleUniqueRepeatingTask(0.2, self.SendNextResponse, self)
	end
end

function GroupCalendar._Whisper:SendNextResponse()
	GroupCalendar:DebugTable("ResponseQueue", self.ResponseQueue)
	
	if #self.ResponseQueue > 0 then
		local vResponse = table.remove(self.ResponseQueue, 1)
		SendChatMessage("<GC> "..vResponse.Message, "WHISPER", nil, vResponse.Name)
	end

	if #self.ResponseQueue == 0 then
		MCSchedulerLib:UnscheduleTask(self.SendNextResponse, self)
	end
end

function GroupCalendar._Whisper:FindEventByIDString(pPlayerName, pEventIDString)
	local vStartIndex, vEndIndex, vPlayerName, vEventID = string.find(pEventIDString, "([^\.]*)\.?([^\.]*)")
	
	if not vPlayerName then
		self:WhisperResponse(pPlayerName, "Unable to decipher the event ID %s", pEventIDString)
		return
	end
	
	if not vEventID or vEventID == "" then
		vEventID = vPlayerName
		vPlayerName = GroupCalendar.PlayerName
	end
	
	vEventID = tonumber(vEventID)
	
	local vDatabase = GroupCalendar.Database.GetDatabase(vPlayerName, false, GroupCalendar.RealmName, true)
	
	if not vDatabase then
		self:WhisperResponse(pPlayerName, "Database for %s not found", vPlayerName)
		return
	end
	
	if not GroupCalendar.Database.DatabaseIsVisible(vDatabase) then
		self:WhisperResponse(pPlayerName, "Access to events for %s is denied", vDatabase.UserName)
		return
	end
	
	--
	
	local vEvent, vDate = GroupCalendar.Database.FindEventByID(vDatabase, vEventID)

	if not vEvent then
		self:WhisperResponse(pPlayerName, "No event with an ID of %d was found in the database for %s", vDatabase.UserName)
	end
	
	return vDatabase, vDate, vEvent
end

function GroupCalendar._Whisper:HandleAttendanceRequest(pEventIDString, pPlayerName, pStatus, pComment)
	local vDatabase, vDate, vEvent = self:FindEventByIDString(pPlayerName, pEventIDString)
	
	if not vEvent then
		return -- Return silently since FindEventByIDString handles it's own error reporting
	end
	
	local vMemberInfo = GroupCalendar.Network:GetGuildMemberInfo(pPlayerName)
	
	if not vMemberInfo then
		self:WhisperResponse(pPlayerName, "Event signups for players outside the guild are not supported yet")
	end
	
	local vRSVP = GroupCalendar.Database.CreatePlayerRSVP(
						vDatabase, vEvent,
						pPlayerName,
						"?", vMemberInfo.Class, vMemberInfo.Level, -- Race, Class, Level
						GroupCalendar.GetMemberDefaultRole(pPlayerName), -- Role
						pStatus,
						pComment,
						nil, nil, -- Guild and rank
						nil) -- Alts
	
	GroupCalendar.Database.AddRSVP(vDatabase, vRSVP)
	
	local vEventRSVP = GroupCalendar.Database.FindEventRSVP(vDatabase.UserName, vEvent, pPlayerName)
	
	if not vEventRSVP then
		self:WhisperResponse(pPlayerName, "Request failed for an unknown reason.  Have the organizer add you manually instead.")
		return
	end
	
	local vEventName = GroupCalendar.Database.GetEventDisplayName(vEvent)
	local vDateString = GroupCalendar.GetLongDateString(vDate, true)
	local vStatusString = GroupCalendar.cStatusCodeStrings[vEventRSVP.mStatus] or "unknown"
	
	if vEvent.mTime then
		local vTimeString = GroupCalendar.GetShortTimeString(vEvent.mTime)
		
		self:WhisperResponse(pPlayerName, "Your status for %s on %s at %s is now %s", vEventName, vDateString, vTimeString, vStatusString)
	else
		self:WhisperResponse(pPlayerName, "Your status for %s on %s is now %s", vEventName, vDateString, vStatusString)
	end
end

----------------------------------------
-- Commands
----------------------------------------

function GroupCalendar._Whisper.Commands:HELP(pPlayerName)
	for _, vHelpString in ipairs(GroupCalendar.cWhisperHelp) do
		self:WhisperResponse(pPlayerName, vHelpString)
	end
end

function GroupCalendar._Whisper.Commands:SUMMARY(pPlayerName)
	local vEventSummary = self:GetEventSummary(pPlayerName)
	local vDate
	local vNumEventsDisplayed = 0
	
	if #vEventSummary > 0 then
		for vIndex, vEventInfo in ipairs(vEventSummary) do
			local vDatabase = GroupCalendar.Database.GetDatabase(vEventInfo.mOwner, false, vEventInfo.mRealm)
			
			if GroupCalendar.Database.DatabaseIsVisible(vDatabase) then
				-- Output a new date header if it's changing
				
				if vEventInfo.mEvent.mDate ~= vDate then
					local vDateString = GroupCalendar.GetLongDateString(vEventInfo.mEvent.mDate, true)
					
					vDate = vEventInfo.mEvent.mDate
					
					self:WhisperResponse(pPlayerName, format(GroupCalendar.cWhisperDateHeader, vDateString))
				end
				
				--
				
				local vTime = GroupCalendar.GetShortTimeString(vEventInfo.mEvent.mTime)
				local vName = GroupCalendar.Database.GetEventDisplayName(vEventInfo.mEvent)
				local vEventRSVP = GroupCalendar.Database.FindEventRSVP(vEventInfo.mOwner, vEventInfo.mEvent, pPlayerName)
				local vStatusString
				local vFormat
				local vEventID
				
				if vEventRSVP then
					vStatusString = GroupCalendar.cStatusCodeStrings[vEventRSVP.mStatus] or "unknown"
				end
				
				if vDatabase.IsPlayerOwned then
					if vDatabase.UserName == GroupCalendar.PlayerName then
				 		vEventID = vEventInfo.mEvent.mID
					else
						vEventID = vDatabase.UserName.."."..vEventInfo.mEvent.mID
					end
					
					if vStatusString then
						vFormat = GroupCalendar.cWhisperOwnedEventStatus
					else
						vFormat = GroupCalendar.cWhisperOwnedEvent
					end
				else
					if vStatusString then
						vFormat = GroupCalendar.cWhisperEventStatus
					else
						vFormat = GroupCalendar.cWhisperEvent
					end
				end
				
				if vStatusString then
					self:WhisperResponse(pPlayerName, vFormat, vTime, vName, vStatusString, vEventID)
				else
					self:WhisperResponse(pPlayerName, vFormat, vTime, vName, vEventID)
				end
				
				vNumEventsDisplayed = vNumEventsDisplayed + 1
			end
		end
	end
	
	if vNumEventsDisplayed == 0 then
		self:WhisperResponse(pPlayerName, GroupCalendar.cWhisperNoEvents)
	end
end

function GroupCalendar._Whisper.Commands:YES(pPlayerName, pEventIDString)
	self:HandleAttendanceRequest(pEventIDString, pPlayerName, "Y")
end

function GroupCalendar._Whisper.Commands:NO(pPlayerName, pEventIDString)
	self:HandleAttendanceRequest(pEventIDString, pPlayerName, "N")
end

function GroupCalendar._Whisper.Commands:STANDBY(pPlayerName, pEventIDString)
	self:HandleAttendanceRequest(pEventIDString, pPlayerName, "S")
end

----------------------------------------
--
----------------------------------------

GroupCalendar.Whisper = GroupCalendar.NewObject(GroupCalendar._Whisper)
